from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class JIssuesConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "jissues"
    verbose_name = _("jissues.app.verbose.name")
